/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: carol@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: ServerExampleActivator.java 2000 2008-08-07 13:42:43Z eyindanga $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.example.server;

import java.net.URL;
import java.rmi.registry.LocateRegistry;
import java.util.HashMap;
import java.util.Hashtable;

import javax.naming.InitialContext;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.ow2.cmi.admin.CMIAdmin;
import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.rmi.JNDIConfig;
import org.ow2.cmi.test.TestItf;
import org.ow2.util.execution.ExecutionResult;
import org.ow2.util.execution.IExecution;
import org.ow2.util.execution.helper.RunnableHelper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

public class ServerExampleActivator implements BundleActivator {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(ServerExampleActivator.class);

    protected static HashMap<String, TestItf> testItfs = new HashMap<String, TestItf>();


    /**
     * Current clusterview manager.
     */
    ClusterViewManager clusterViewManager = null;

    /*
     * (non-Javadoc)
     *
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    public void start(final BundleContext arg0) throws Exception {
        try {
            // testIIOPServer();
            testJRMPServer();
        } catch (Exception e) {
            // TODO: handle exception
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext arg0) throws Exception {
        // TODO Auto-generated method stub
        clusterViewManager = null;

    }


    /**
     * Sample JRMP server
     */
    public void testJRMPServer() {
        // Get the rmi port property
        String rmiPortString = System.getProperty(
                ServerExampleProperty.RMI_PORT_KEY.getPropertyName(), String
                        .valueOf(ServerExampleProperty.DEFAULT_RMI_PORT
                                .getPropertyName()));

        // Get the rmi port property
        final String confFileName = System.getProperty(
                ServerExampleProperty.CONF_FILE_URL_KEY.getPropertyName(),
                String.valueOf(ServerExampleProperty.DEFAULT_CONF_FILE_URL
                        .getPropertyName()));
        // Use integer value
        final int rmiPort = Integer.valueOf(rmiPortString);
        logger.info("Starting JRMP Server");
        try {

            IExecution<Void> registryExec = new IExecution<Void>() {

                private CMIAdmin cmiAdmin;

                public Void execute() throws Exception {
                    LocateRegistry.createRegistry(rmiPort);
                    logger.info("Registry started");

                    URL url = getClass().getClassLoader().getResource(
                            confFileName);

                    Hashtable<String, ?> env = JNDIConfig.getCMIEnv(url);
                    new InitialContext(env);

                    cmiAdmin = CMIAdmin.getCMIAdmin();
                    return null;
                }
            };
            // Execute
            RunnableHelper<Void> nsHelper = new RunnableHelper<Void>();

            ExecutionResult<Void> nsStartExecResult = nsHelper.execute(
                    getClass().getClassLoader(), registryExec);

            if (nsStartExecResult.hasException()) {
                logger.error("Unable to start CMI ", nsStartExecResult
                        .getException());
                return;
            }

        } catch (Exception e) {
            // TODO Auto-generated catch block
            logger.error("Unable to start the CMI registry ", e);
            return;
        }

    }

}
